<?php

/**
 * @file
 * Verify IPN updates from BitPay and update relevant transactions.
 */

/**
 * Interpret IPN updates from BitPay and update relevant payment transactions.
 */
function commerce_bitpay_verify_ipn() {
  $payment_method = commerce_payment_method_instance_load('bitpay|commerce_payment_bitpay');
  $payment_method['settings'] += commerce_bitpay_default_settings();

  libraries_load('php-client');

  $json = bpVerifyNotification($payment_method['settings']['apiKey']);

  if (is_null($json) || is_string($json) || !isset($json['posData']) || !is_numeric($json['posData'])) {
    switch ($json) {
      case 'No post data':
        watchdog('commerce_bitpay', 'IPN POST had no data.', array(), WATCHDOG_WARNING);
        break;

      case 'no posData':
        watchdog('commerce_bitpay', 'No IPN posData value was provided.', array(), WATCHDOG_WARNING);
        break;

      case 'authentication failed (bad hash)':
        watchdog('commerce_bitpay', 'IPN verification failed (bad hash).', array(), WATCHDOG_WARNING);
        break;

      default:
        watchdog('commerce_bitpay', 'IPN posted something strange: @string', array('@string' => $json));
    }
    return FALSE;
  }

  $order_id = db_select('commerce_payment_transaction', 'cpt')
    ->fields('cpt', array('order_id'))
    ->condition('transaction_id', $json['posData'])
    ->range(0, 1)
    ->addTag('commerce_bitpay_verify_ipn')
    ->execute()
    ->fetchField();
  if (empty($order_id) || !is_numeric($order_id)) {
    watchdog('commerce_bitpay', 'Failed to retrieve an order ID for IPN.');
    return FALSE;
  }
  watchdog('commerce_bitpay', 'IPN received for order @order_id, transaction @trans_id.', array('@trans_id' => $json['posData'], '@order_id' => $order_id), WATCHDOG_DEBUG, l(t('view order payments'), 'admin/commerce/orders/' . $order_id . '/payment'));

  commerce_bitpay_update_transaction($json['posData'], $order_id, $json['status']);
}
